import sys

# True if we are running on Python 3.
PY3 = sys.version_info[0] == 3

if PY3:  # pragma: no cover
    from urllib import parse
    urlparse = parse
    from urllib.parse import quote as url_quote
    from urllib.parse import urlencode as url_encode
else:  # pragma: no cover
    import urlparse
    from urllib import quote as url_quote
    from urllib import urlencode as url_encode



import os
import unittest

from pyramid_sna import read_setting_from_env


class CallbackCounter(object):

    def __init__(self):
        self.reset()

    def __call__(self, *args, **kwargs):
        self.calls.append({'args': args, 'kwargs': kwargs})
        return 'Register success!'

    @property
    def counter(self):
        return len(self.calls)

    def reset(self):
        self.calls = []


class ConfigTests(unittest.TestCase):

    def test_read_setting_from_env(self):
        settings = {
            'foo_bar': '1',
        }

        self.assertEqual('1', read_setting_from_env(settings, 'foo_bar'))

        self.assertEqual('default',
                         read_setting_from_env(settings, 'new_option', 'default'))

        self.assertEqual(None,
                         read_setting_from_env(settings, 'new_option'))

        os.environ['FOO_BAR'] = '2'
        self.assertEqual('2', read_setting_from_env(settings, 'foo_bar'))



from pyramid_sna.oauth2 import get_user_info, oauth2_step1, oauth2_step2


def google_login(request):
    settings = request.registry.settings
    return oauth2_step1(
        request=request,
        auth_uri=settings['google_auth_uri'],
        client_id=settings['google_client_id'],
        redirect_url=request.route_url('google_callback'),
        scope=settings['google_scope'],
    )


def google_callback(request):
    settings = request.registry.settings
    access_token = oauth2_step2(
        request=request,
        token_uri=settings['google_token_uri'],
        client_id=settings['google_client_id'],
        client_secret=settings['google_client_secret'],
        redirect_url=request.route_url('google_callback'),
        scope=settings['google_scope'],
    )

    info = get_user_info(settings['google_user_info_uri'], access_token)
    user_id = info['id']
    new_info = {
        'screen_name': info.get('name', ''),
        'first_name': info.get('given_name', ''),
        'last_name': info.get('family_name', ''),
        'email': info.get('email', ''),
    }

    return settings['google_callback'](request, user_id, new_info)



import unittest

from mock import patch
from webtest import TestApp

from pyramid import testing
from pyramid.request import Request

from pyramid_sna import main, include_google
from pyramid_sna.compat import urlparse
from pyramid_sna.tests import CallbackCounter


def example_callback(*args, **kwargs):
    pass  # pragma: no cover


class GoogleConfigTests(unittest.TestCase):

    def test_incomplete_config1(self):
        config = testing.setUp(settings={})
        self.assertFalse(include_google(config))
        testing.tearDown()

    def test_incomplete_config2(self):
        config = testing.setUp(settings={
            'google_client_id': '123',
        })
        self.assertFalse(include_google(config))
        testing.tearDown()

    def test_incomplete_config3(self):
        config = testing.setUp(settings={
            'google_client_id': '123',
            'google_client_secret': 's3cr3t',
        })
        self.assertFalse(include_google(config))
        testing.tearDown()

    def test_complete_config(self):
        config = testing.setUp(settings={
            'google_client_id': '123',
            'google_client_secret': 's3cr3t',
            'google_callback': example_callback,
        })
        self.assertTrue(include_google(config))
        testing.tearDown()


class GoogleViewTests(unittest.TestCase):

    def setUp(self):
        self.callback_counter = CallbackCounter()
        settings = {
            'google_client_id': '123',
            'google_client_secret': 's3cr3t',
            'google_callback': self.callback_counter,
            'google_scope': 'perm1 perm2',
        }
        app = main({}, **settings)
        self.testapp = TestApp(app)

    def tearDown(self):
        self.testapp.reset()

    def test_google_login(self):
        res = self.testapp.get('/google/login', {
            'next_url': 'https://localhost/foo/bar',
        })
        self.assertEqual(res.status, '302 Found')
        url = urlparse.urlparse(res.location)
        self.assertEqual(url.netloc, 'accounts.google.com')
        self.assertEqual(url.path, '/o/oauth2/auth')
        query = urlparse.parse_qs(url.query)
        self.assertEqual(sorted(query.keys()), [
            'client_id', 'redirect_uri', 'response_type', 'scope', 'state',
        ])
        self.assertEqual(query['scope'], ['perm1 perm2'])
        self.assertEqual(query['redirect_uri'],
                         ['http://localhost/google/callback'])
        self.assertEqual(query['client_id'], ['123'])

    def test_google_callback(self):
        # call the login to fill the session
        res = self.testapp.get('/google/login', {
            'next_url': 'https://localhost/foo/bar',
        })
        self.assertEqual(res.status, '302 Found')
        url = urlparse.urlparse(res.location)
        query = urlparse.parse_qs(url.query)
        state = query['state'][0]

        with patch('requests.post') as fake_post:
            fake_post.return_value.status_code = 200
            fake_post.return_value.json = lambda: {
                'access_token': '1234',
            }
            with patch('requests.get') as fake_get:
                fake_get.return_value.status_code = 200
                fake_get.return_value.json = lambda: {
                    'id': '789',
                    'name': 'John Doe',
                    'given_name': 'John',
                    'family_name': 'Doe',
                    'email': 'john@example.com',
                }

                res = self.testapp.get('/google/callback', {
                    'code': '1234',
                    'state': state,
                })
                self.assertEqual(res.status, '200 OK')
                self.assertEqual(res.text, 'Register success!')

                self.assertEqual(self.callback_counter.counter, 1)
                call_args = self.callback_counter.calls[0]['args']
                self.assertTrue(isinstance(call_args[0], Request))
                self.assertEqual(call_args[1], '789')
                self.assertEqual(call_args[2], {
                    'email': 'john@example.com',
                    'first_name': 'John',
                    'last_name': 'Doe',
                    'screen_name': 'John Doe',
                })
                call_kwargs = self.callback_counter.calls[0]['kwargs']
                self.assertEqual(call_kwargs, {})



import os
from pyramid.config import Configurator


def main(global_config, **settings):
    config = Configurator(settings=settings)
    config.include('pyramid_beaker')
    includeme(config)

    return config.make_wsgi_app()


def read_setting_from_env(settings, key, default=None):
    env_variable = key.upper()
    if env_variable in os.environ:
        return os.environ[env_variable]
    else:
        return settings.get(key, default)


def include_google(config):
    settings = config.registry.settings

    for key, default in (
        ('client_id', None),
        ('client_secret', None),
        ('callback', None),
        ('auth_uri', 'https://accounts.google.com/o/oauth2/auth'),
        ('token_uri', 'https://accounts.google.com/o/oauth2/token'),
        ('user_info_uri', 'https://www.googleapis.com/oauth2/v1/userinfo'),
        ('scope', 'https://www.googleapis.com/auth/userinfo.email https://www.googleapis.com/auth/userinfo.profile'),
    ):

        option = 'google_%s' % key
        settings[option] = read_setting_from_env(settings, option, default)

    if (
        settings['google_client_id']
        and settings['google_client_secret']
        and settings['google_callback']
    ):

        config.add_route('google_login', '/google/login')
        config.add_view('pyramid_sna.google.google_login',
                        route_name='google_login',
                        renderer='string')

        config.add_route('google_callback', '/google/callback')
        config.add_view('pyramid_sna.google.google_callback',
                        route_name='google_callback',
                        renderer='string')

        callback = config.maybe_dotted(settings['google_callback'])
        settings['google_callback'] = callback

        return True
    else:
        return False


def include_facebook(config):
    settings = config.registry.settings

    for key, default in (
        ('app_id', None),
        ('app_secret', None),
        ('callback', None),
        ('dialog_oauth_url', 'https://www.facebook.com/dialog/oauth/'),
        ('access_token_url', 'https://graph.facebook.com/oauth/access_token'),
        ('basic_information_url', 'https://graph.facebook.com/me'),
        ('scope', 'email'),
    ):
        option = 'facebook_%s' % key
        settings[option] = read_setting_from_env(settings, option, default)

    if (
        settings['facebook_app_id']
        and settings['facebook_app_secret']
        and settings['facebook_callback']
    ):
        config.add_route('facebook_login', '/facebook/login')
        config.add_view('pyramid_sna.facebook.facebook_login',
                        route_name='facebook_login',
                        renderer='string')

        config.add_route('facebook_callback', '/facebook/callback')
        config.add_view('pyramid_sna.facebook.facebook_callback',
                        route_name='facebook_callback',
                        renderer='string')

        callback = config.maybe_dotted(settings['facebook_callback'])
        settings['facebook_callback'] = callback

        return True
    else:
        return False


def include_liveconnect(config):
    settings = config.registry.settings

    for key, default in (
        ('client_id', None),
        ('client_secret', None),
        ('callback', None),
        ('auth_uri', 'https://login.live.com/oauth20_authorize.srf'),
        ('token_uri', 'https://login.live.com/oauth20_token.srf'),
        ('basic_information_url', 'https://apis.live.net/v5.0/me'),
        ('scope', 'wl.basic wl.emails'),
    ):
        option = 'liveconnect_%s' % key
        settings[option] = read_setting_from_env(settings, option, default)

    if (
        settings['liveconnect_client_id']
        and settings['liveconnect_client_secret']
        and settings['liveconnect_callback']
    ):
        config.add_route('liveconnect_login', '/liveconnect/login')
        config.add_view('pyramid_sna.liveconnect.liveconnect_login',
                        route_name='liveconnect_login',
                        renderer='string')

        config.add_route('liveconnect_callback', '/liveconnect/callback')
        config.add_view('pyramid_sna.liveconnect.liveconnect_callback',
                        route_name='liveconnect_callback',
                        renderer='string')

        callback = config.maybe_dotted(settings['liveconnect_callback'])
        settings['liveconnect_callback'] = callback

        return True
    else:
        return False


def includeme(config):
    settings = config.registry.settings
    settings['google_auth_enabled'] = include_google(config)
    settings['facebook_auth_enabled'] = include_facebook(config)
    settings['liveconnect_auth_enabled'] = include_liveconnect(config)



from pyramid_sna.oauth2 import get_user_info, oauth2_step1, oauth2_step2


def liveconnect_login(request):
    settings = request.registry.settings
    return oauth2_step1(
        request=request,
        auth_uri=settings['liveconnect_auth_uri'],
        client_id=settings['liveconnect_client_id'],
        redirect_url=request.route_url('liveconnect_callback'),
        scope=settings['liveconnect_scope'],
    )


def liveconnect_callback(request):
    settings = request.registry.settings
    access_token = oauth2_step2(
        request=request,
        token_uri=settings['liveconnect_token_uri'],
        client_id=settings['liveconnect_client_id'],
        client_secret=settings['liveconnect_client_secret'],
        redirect_url=request.route_url('liveconnect_callback'),
        scope=settings['liveconnect_scope'],
    )

    info = get_user_info(settings['liveconnect_basic_information_url'],
                         access_token)
    user_id = info['id']
    info['screen_name'] = info.get('name', '')
    if 'wl.emails' in settings['liveconnect_scope']:
        info['email'] = info.get('emails', {}).get('account', '')

    return settings['liveconnect_callback'](request, user_id, info)
